/*===-- PapiProfiling.c - Support library for papi profiling --------------===*\
|*
|*                     The LLVM Compiler Infrastructure
|*
|* This file is distributed under the University of Illinois Open Source
|* License. See LICENSE.TXT for details.
|*
|*===----------------------------------------------------------------------===*|
|*
|* This file implements the call back routines for the papi profiling
|* instrumentation pass.  This should be used with the -insert-papi-profiling
|* LLVM pass.
|*
\*===----------------------------------------------------------------------===*/

#include "Profiling.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

static long long *FunctionCounters;
static long long *RegionCounters;
static unsigned NumFunctions;
static unsigned NumRegions;

static void PapiProfAtExitHandler(void) {
  /* Note that if this were doing something more intelligent with the
   * instrumentation, we could do some computation here to expand what we
   * collected into simple edge profiles.  Since we directly count each edge, we
   * just write out all of the counters directly.
   */
  write_profiling_data64(PapiInfo, FunctionCounters, NumFunctions);
}

static void PapiRegionProfAtExitHandler(void) {
  write_profiling_data64(PapiRegionInfo, RegionCounters, NumRegions);
}

int llvm_start_papi_profiling(int argc, const char **argv,
                              long long *arrayStart, unsigned numElements) {
  int Ret = save_arguments(argc, argv);
  FunctionCounters = arrayStart;
  NumFunctions = numElements;
  atexit(PapiProfAtExitHandler);
  return Ret;
}

int llvm_start_papi_region_profiling(int argc, const char **argv,
                              long long *arrayStart, unsigned numElements) {
  int Ret = save_arguments(argc, argv);
  RegionCounters = arrayStart;

  memset(arrayStart, 0, sizeof(long long)*numElements);

  NumRegions = numElements;
  atexit(PapiRegionProfAtExitHandler);
  return Ret;
}
